# Copyright (c) 2008-2011 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.directory.form
import gocept.directory.interfaces
import grok
import megrok.pagelet
import z3c.flashmessage.interfaces
import zope.app.authentication.httpplugins
import zope.app.security.interfaces
import zope.component
import zope.dublincore.interfaces
import zope.pluggableauth.authentication
import zope.pluggableauth.plugins.httpplugins
import zope.pluggableauth.interfaces
import gocept.directory.auth


class ManageDirectory(grok.Permission):
    grok.name('gocept.directory.ManageDirectory')


def setup_authentication(pau):
    pau.credentialsPlugins = ('basic-auth',)
    pau.authenticatorPlugins = ('machine-logins',)


grok.global_utility(zope.pluggableauth.plugins.httpplugins.HTTPBasicAuthCredentialsPlugin,
                    provides=zope.pluggableauth.interfaces.ICredentialsPlugin,
                    name='basic-auth')
grok.global_utility(gocept.directory.auth.MachineLogin,
                    provides=zope.pluggableauth.interfaces.IAuthenticatorPlugin,
                    name='machine-logins')


class Directory(grok.Application, grok.Container):

    grok.local_utility(
        zope.pluggableauth.authentication.PluggableAuthentication,
        provides=zope.app.security.interfaces.IAuthentication,
        setup=setup_authentication)

    zope.interface.implements(gocept.directory.interfaces.IDirectory)

    @property
    def locations(self):
        for obj in self.values():
            if gocept.directory.interfaces.ILocation.providedBy(obj):
                yield obj

    @property
    def vlan_classes(self):
        for obj in self.values():
            if gocept.directory.interfaces.IVLANClass.providedBy(obj):
                yield obj

    @property
    def nodes(self):
        for rg in self.resource_groups:
            for node in rg.nodes:
                yield node

    @property
    def networks(self):
        for location in self.locations:
            for vlan in location.vlans:
                for network in vlan.networks:
                    yield network

    @property
    def resource_groups(self):
        for obj in self.values():
            if gocept.directory.interfaces.IResourceGroup.providedBy(obj):
                yield obj

    @property
    def groups(self):
        for obj in self.values():
            if gocept.directory.interfaces.IGroup.providedBy(obj):
                yield obj

    @property
    def people(self):
        for obj in self.values():
            if gocept.directory.interfaces.IPerson.providedBy(obj):
                yield obj

    def node_by_name(self, name):
        for node in self.nodes:
            if node.name == name:
                return node
        raise KeyError(name)

    def location_by_name(self, name):
        for location in self.locations:
            if location.abbreviation == name:
                return location
        raise KeyError(name)

    def password_by_service(self, name):
        return self['pw-' + name]

    def passwords(self):
        for obj in self.values():
            if gocept.directory.interfaces.IPassword.providedBy(obj):
                return obj


def locate_directory_from(current):
    directory = None
    while current is not None:
        if gocept.directory.interfaces.IDirectory.providedBy(current):
            return current
        current = current.__parent__
    raise ValueError('cannot locate directory object')


class Index(megrok.pagelet.Pagelet):

    grok.require('gocept.directory.ManageDirectory')


class Edit(gocept.directory.form.BasicEditForm):
    pass


class Layout(megrok.pagelet.Layout):
    grok.context(zope.interface.Interface)

    megrok.pagelet.template('layout.pt')


class LayoutHelper(grok.View):
    grok.context(zope.interface.Interface)

    def render(self):
        return ''

    def messages(self):
        receiver = zope.component.getUtility(
            z3c.flashmessage.interfaces.IMessageReceiver)
        return list(receiver.receive())

