# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import grok
import megrok.pagelet
import gocept.directory.interfaces
import zope.component


class Location(grok.Container):

    grok.implements(gocept.directory.interfaces.ILocation)

    @property
    def directory(self):
        return self.__parent__

    @property
    def vlans(self):
        for obj in self.values():
            if gocept.directory.interfaces.IVLAN.providedBy(obj):
                yield obj

    @property
    def nodes(self):
        for rg in self.directory.resource_groups:
            for node in rg.nodes:
                if node.location is self:
                    yield node


class Index(megrok.pagelet.Pagelet):

    grok.require('gocept.directory.ManageDirectory')


class AddLocation(gocept.directory.form.AddForm):

    grok.context(gocept.directory.interfaces.IDirectory)
    form_fields = grok.AutoFields(gocept.directory.interfaces.ILocation)

    @grok.action('Add')
    def add(self, **kw):
        location = Location()
        self.applyData(location, **kw)
        self.context[kw['abbreviation']] = location
        self.status = u'Created location %s' % kw['title']
        self.redirect(self.url(location))


class Rollout(megrok.pagelet.Pagelet):
    """Helper to perform environment roll-outs."""

    grok.require('gocept.directory.ManageDirectory')


class BackToProduction(megrok.pagelet.Pagelet):

    safety_belt = False

    def update(self):
        self.nodes = []
        self.safety_belt = 'safetybelt' in self.request.form
        if not self.safety_belt:
            return
        for node in self.context.nodes:
            if node.environment == 'staging':
                node.environment = 'production'
                self.nodes.append(node.__name__)

    def render(self):
        if not self.safety_belt:
            self.flash('Safety belt not applied!')
        else:
            self.flash('Updated nodes: %s' % (', '.join(self.nodes)))
        self.request.response.redirect(self.url(self.context, name='rollout'))


class RolloutSelection(megrok.pagelet.Pagelet):

    grok.require('gocept.directory.ManageDirectory')

    def update(self):
        self.selection = set()
        for node in getattr(self, 'strategy_%s' %
                            self.request.form['strategy'])():
            self.selection.add(node)

    @property
    def candidates(self):
        return list(node for node in self.context.nodes
                    if node.environment == 'production')

    def strategy_core(self):
        for candidate in self.candidates:
            if 'role::puppetmaster' in candidate.puppet_classes:
                yield candidate
            if 'role::portage_binhost' in candidate.puppet_classes:
                yield candidate
            if 'test' == candidate.resource_group.name:
                yield candidate

    def strategy_remaining(self):
        amount = int(self.request.get('amount', 10))
        return list(self.candidates)[:amount]

    def strategy_representative(self):
        required_classes = set()
        for node in self.candidates:
            required_classes.update(node.puppet_classes)
        for node in self.context.nodes:
            if node.environment != 'staging':
                continue
            required_classes.difference_update(node.puppet_classes)

        while required_classes:
            max_node, max_count = None, 0
            for node in self.candidates:
                this_count = len(node.puppet_classes.intersection(required_classes))
                if this_count > max_count:
                    max_node = node
                    max_count = this_count

            yield max_node
            required_classes.difference_update(max_node.puppet_classes)


class RolloutDo(megrok.pagelet.Pagelet):

    grok.require('gocept.directory.ManageDirectory')

    def update(self):
        self.nodes = self.request.form['nodes']
        if isinstance(self.nodes, unicode):
            self.nodes = [self.nodes]
        for node_name in self.nodes:
            node = zope.component.getUtility(
                gocept.directory.interfaces.INode, name=node_name)
            node.environment = 'staging'

    def render(self):
        self.flash('Updated nodes: %s' % (', '.join(self.nodes)))
        self.request.response.redirect(self.url(self.context, name='rollout'))
