# Copyright (c) 2010 gocept gmbh & co. kg
# See also LICENSE.txt

import unittest
import gocept.directory.app
import gocept.directory.nic
import gocept.directory.testing
import zope.app.testing.functional


class NICTests(zope.app.testing.functional.FunctionalTestCase):

    layer = gocept.directory.testing.DataLayer

    def setUp(self):
        super(NICTests, self).setUp()

        self.directory = self.getRootFolder()['directory']
        self.oldsite = zope.app.component.hooks.getSite()
        zope.app.component.hooks.setSite(self.directory)

        self.directory['jamaica'] = gocept.directory.location.Location()

        self.directory['rg'] = self.rg = \
                gocept.directory.resourcegroup.ResourceGroup()

    def tearDown(self):
        zope.app.component.hooks.setSite(self.oldsite)
        super(NICTests, self).tearDown()

    def test_mac_consistency_no_nics(self):
        check = gocept.directory.nic.MACConsistency(self.directory)
        self.assertEquals([], check.errors)

    def test_mac_consistency_one_nic(self):
        self.rg['node1'] = node1 = gocept.directory.nodes.Node()
        node1.name = 'node1'
        node1['fe'] = gocept.directory.nic.NIC()
        node1['fe'].vlan = self.directory['jamaica']['vlan-fe']
        node1['fe'].mac = '1'

        check = gocept.directory.nic.MACConsistency(self.directory)
        self.assertEquals([], check.errors)

    def test_mac_consistency_two_nics(self):
        self.rg['node1'] = node1 = gocept.directory.nodes.Node()
        node1.name = 'node1'
        node1['fe'] = gocept.directory.nic.NIC()
        node1['fe'].vlan = self.directory['jamaica']['vlan-fe']
        node1['fe'].mac = '1'
        self.rg['node2'] = node2 = gocept.directory.nodes.Node()
        node2.name = 'node2'
        node2['fe'] = gocept.directory.nic.NIC()
        node2['fe'].vlan = self.directory['jamaica']['vlan-fe']
        node2['fe'].mac = '2'

        check = gocept.directory.nic.MACConsistency(self.directory)
        self.assertEquals([], check.errors)

        # Make the second node conflict
        node2['fe'].mac = '1'

        check = gocept.directory.nic.MACConsistency(self.directory)
        self.assertEquals(['Duplicate MAC: 1 by node1.fe, node2.fe'], check.errors)
